"""
dipole_cube_main:
interactive exploration of the dipole configuration described by
Hartung et al, PRB, and Refs cited therein.
Ingo Rehberg, Feb 3, 2020, tested with PY 3.7.5
"""
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.widgets import Slider, RadioButtons, Button
from mpl_toolkits.mplot3d import axes3d  # is used for ax1, line 26
from dipole_cube_modules import dipole_cube_setting
from dipole_cube_modules import dipole_cube_examination_geometry
from dipole_cube_modules import dipole_cube_figures as dcf
# --initial settings for interactively adjustable parameters:
tau = 90  # rotation angle (degree) of the (1,1,1)-dipole around space diagonal
trp = 0.5  # transparency of the sphere, adjustable with slider
sphere_resol = 10  # resolution of the sphere, adjustable with button
display_plane = False  # RadioButton for ax1
display_sphere = False    # RadioButton for ax1
display_diag = True  # RadioButton for ax1
display_edge = True  # RadioButton for ax1
loglog = True  # RadioButton for ax2
# ----------------------------------------------------------------------------
# instance for the examination line and a plane perpendicular to it:
g = dipole_cube_examination_geometry.Geo(richtung=np.array([1., 1., 0.]),
                                         origin=np.array([0, 0., 0]),
                                         length=10.,
                                         d_p=2, width=2, st=0.1)
mycube = dipole_cube_setting.Cuboid(90.)  # position and orientation: 8 dipoles
# layout for 7 plots and sliders etc:
fig = plt.figure(figsize=(8.*5./3., 8.))  # 7 subplots,
qx = 0.15  # width of 5 plots
qy = qx*5./3.  # height of that plots
left2 = 0.37  # left anchor for positions
ax1 = plt.axes([0.01, 0.3, 0.3, 0.3*5./3.], projection='3d')  # 3-d animation
ax2 = plt.axes([0.37, 0.2, 0.35, 0.3])  # 3 B-components along "direction"
ax3 = plt.axes([left2, 0.7, qx, qy])  # Energy
ax4 = plt.axes([left2+1.4*qx, 0.7, qx, qy])  # Potential
ax5 = plt.axes([left2+2*1.4*qx, 0.7, qx, qy])  # B_perpendicular
ax6 = plt.axes([left2+2*1.4*qx, 0.4, qx, qy])  # B_upwards
ax7 = plt.axes([left2+2*1.4*qx, 0.1, qx, qy])  # B_horizontal
tau_neu = Slider(plt.axes([0.02, 0.05, 0.28, 0.03], fc='lightgoldenrodyellow'),
                 r"$\tau$", 0, 360, valfmt='%1.0f', valinit=tau)
trp_neu = Slider(plt.axes([0.02, 0.1, 0.28, 0.03], fc='lightgoldenrodyellow'),
                 "$\\alpha$", 0, 1, valfmt='%1.1f', valinit=trp)
y_neu = Slider(plt.axes([left2, 0.1, 0.14, 0.02], fc='lightgoldenrodyellow'),
               "$ y $", 0, 5, valfmt='%1.1f', valinit=g.richtung[1])
z_neu = Slider(plt.axes([left2, 0.05, 0.14, 0.02], fc='lightgoldenrodyellow'),
               "$ z $", 0, 5, valfmt='%1.1f', valinit=g.richtung[2])
dp_neu = Slider(plt.axes([left2+0.22, 0.62, 0.14, 0.02],
                         fc='lightgoldenrodyellow'), r"$d_\mathrm{p}$",
                0, 4, valfmt='%1.1f', valinit=g.d_p)
length_neu = Slider(plt.axes([left2+0.22, 0.57, 0.14, 0.02],
                             fc='lightgoldenrodyellow'), r"$L$",
                    0, 20, valfmt='%1.1f', valinit=g.length)
oy_neu = Slider(plt.axes([left2+0.22, 0.1, 0.14, 0.02],
                         fc='lightgoldenrodyellow'),
                "$ O_y $", 0, 1, valfmt='%1.1f', valinit=g.origin[1])
oz_neu = Slider(plt.axes([left2+0.22, 0.05, 0.14, 0.02],
                         fc='lightgoldenrodyellow'),
                "$ O_z $", 0, 1, valfmt='%1.1f', valinit=g.origin[2])

loglog_toggle = RadioButtons(plt.axes([0.37, 0.51, 0.08, 0.08],
                                      fc='lightgoldenrodyellow'),
                             ('loglog', 'linear'), active=0)
display_cuboid = RadioButtons(plt.axes([0.2, 0.82, 0.09, 0.14],
                                       fc='lightgoldenrodyellow'),
                              ('edge', 'diag', 'both', 'none'), active=2)
display_surface = RadioButtons(plt.axes([0.01, 0.85, 0.09, 0.11],
                                        fc='lightgoldenrodyellow'),
                               ('plane', 'sphere', 'none'), active=2)
decrease_button = Button(plt.axes([0.02, 0.15, 0.04, 0.04]),
                         '<', color='lightgoldenrodyellow', hovercolor='0.975')
increase_button = Button(plt.axes([0.1, 0.15, 0.04, 0.04]),
                         '>', color='lightgoldenrodyellow', hovercolor='0.975')


def display_mw(w):
    fig.text((0.06+0.1)/2., 0.17, str(w),
             horizontalalignment='center', verticalalignment='center',
             bbox=dict(fc='white'))


# rc Params settings:
plt.rc('axes', fc='white', edgecolor='black', titlesize=20, labelsize=12,
       grid=True)  # set the format of the plotaxes
plt.rc('legend', fontsize=22, frameon=False, numpoints=1,
       labelspacing=0.3, columnspacing=0)  # set the format of the legend
plt.rc('xtick', labelsize=12, top=True, direction='in')  # ticklabels
plt.rc('ytick', labelsize=12, right=True, direction='in')
plt.rc('font', family="serif", size=18)
plt.rc('text', usetex=False)

# list of actions triggered by sliders and buttons:----


def update_tau(val):
    global tau
    tau = tau_neu.val
    mycube.set_ang(tau)
    dcf.bild1(ax1, g, mycube, display_plane, display_sphere, display_diag,
              display_edge, trp, sphere_resol)
    dcf.bild2(ax2, g, mycube, loglog)
    dcf.bild3(ax3, g, mycube)
    dcf.bild4(ax4, g, mycube)
    dcf.bild5(ax5, g, mycube)
    dcf.bild6(ax6, g, mycube)
    dcf.bild7(ax7, g, mycube)


def update_trp(val):  # tranparency of the sphere or the plane
    global trp
    trp = trp_neu.val
    dcf.bild1(ax1, g, mycube, display_plane, display_sphere,
              display_diag, display_edge, trp, sphere_resol)


def update_y(val):  # the y-komponent of the display line,
    # which is also perpendicular to the plane
    g.richtung[1] = y_neu.val
    g.new_geo()
    update_tau(tau)


def update_z(val):  # the z-komponent of the display line,
    # which is also perpendicular to the plane
    g.richtung[2] = z_neu.val
    g.new_geo()
    update_tau(tau)


def update_oy(val):  # the y-value of the display line for x=0
    g.origin[1] = oy_neu.val
    g.new_geo()
    update_tau(tau)


def update_oz(val):  # the z-value of the display line for x=0
    g.origin[2] = oz_neu.val
    g.new_geo()
    update_tau(tau)


def update_dp(val):  # the distance of the plane
    g.d_p = dp_neu.val
    g.new_geo()
    update_tau(tau)


def update_length(val):  # the distance of the plane
    g.length = length_neu.val
    g.new_geo()
    update_tau(tau)


def toggle_cuboid(label):  # set/erase additional lines for the cuboid
    global display_diag, display_edge
    display_edge = bool(label == 'edge')
    display_diag = bool(label == 'diag')
    if (label == 'both'):
        display_edge = display_diag = True
    dcf.bild1(ax1, g, mycube, display_plane, display_sphere,
              display_diag, display_edge, trp, sphere_resol)


def toggle_suface(label):  # set/erase additional surfaces (plane or sphere)
    # to display the magnetic induction perpendicular to that surface
    global display_plane, display_sphere
    display_plane = display_sphere = False
    if (label == 'plane'):
        display_plane = True
    elif (label == 'sphere'):
        display_sphere = True
    dcf.bild1(ax1, g, mycube, display_plane, display_sphere,
              display_diag, display_edge, trp, sphere_resol)


def toggle_log(label):  # toggle between linear and loglog presentation
    # of B_x, B_y and B_z
    global loglog
    loglog = bool(label == 'loglog')
    dcf.bild2(ax2, g, mycube, loglog)


def decrease(event):  # decrease sphere resolution (for a speedy presentation)
    global sphere_resol
    if (sphere_resol % 2 == 0):
        sphere_resol /= 2
    else:
        sphere_resol /= 3
    if sphere_resol < 10:
        sphere_resol = 10
    display_mw(sphere_resol)
    dcf.bild1(ax1, g, mycube, display_plane, display_sphere,
              display_diag, display_edge, trp, sphere_resol)


def increase(event):  # increase sphere resolution
    # (presentaion might become very slow)
    global sphere_resol
    sphere_resol *= 2
    if sphere_resol > 360:
        sphere_resol = 360
    display_mw(sphere_resol)
    dcf.bild1(ax1, g, mycube, display_plane, display_sphere,
              display_diag, display_edge, trp, sphere_resol)


tau_neu.on_changed(update_tau)
trp_neu.on_changed(update_trp)
y_neu.on_changed(update_y)
z_neu.on_changed(update_z)
oy_neu.on_changed(update_oy)
oz_neu.on_changed(update_oz)
dp_neu.on_changed(update_dp)
length_neu.on_changed(update_length)
display_cuboid.on_clicked(toggle_cuboid)
display_surface.on_clicked(toggle_suface)
loglog_toggle.on_clicked(toggle_log)
decrease_button.on_clicked(decrease)
increase_button.on_clicked(increase)
# ---------- end of actions triggered by sliders and buttons.--------------
display_mw(sphere_resol)
update_tau(tau)                        # show all 7 subplots
plt.show()
