"""
# dipole_cube_setting.py
# definition and properties of the 8-dipole-cluster
# Ingo Rehberg, Feb 03, 2020
"""
import numpy as np


class Cuboid:
    def __init__(self, tau=90):  # init functions for all constants needed
        self.x, self.y, self.z = np.meshgrid(np.linspace(-1, 1, 2),
                                             np.linspace(-1, 1, 2),
                                             np.linspace(-1, 1, 2))
        self.set_ang(tau)

    def get_xyzuvw(self):
        return self.x, self.y, self.z, self.u, self.v, self.w

    def get_lin_coord(self):
        x = np.linspace(0, 7, 8)
        y = np.linspace(0, 7, 8)
        z = np.linspace(0, 7, 8)
        u = np.linspace(0, 7, 8)
        v = np.linspace(0, 7, 8)
        w = np.linspace(0, 7, 8)
        adr = 0
        for ix in (-1, 1):
            xa = int((ix+1)//2)
            for iy in (-1, 1):
                ya = int((iy+1)//2)
                for iz in (-1, 1):
                    za = int((iz+1)//2)
                    x[adr] = self.x[za][ya][xa]
                    y[adr] = self.y[za][ya][xa]
                    z[adr] = self.z[za][ya][xa]
                    u[adr] = self.u[za][ya][xa]
                    v[adr] = self.v[za][ya][xa]
                    w[adr] = self.w[za][ya][xa]
                    adr += 1
        return x, y, z, u, v, w

    def set_ang(self, tau):  # Make the direction data for the arrows
        self.t = np.pi/180. *tau  # in radian, alpha here =tau in paper
        self.u = - np.sin(self.t - 4./3.*np.pi) * np.sqrt(2./3.)*self.y*self.z    # *self.y*self.z for symmetry
        self.v = -  np.sin(self.t -  2./3.*np.pi)     * np.sqrt(2./3.) * self.x*self.z     # 2./3.*np.pi vector components phase shifted by 120 degrees
        self.w = - np.sin(self.t               )    * np.sqrt(2./3.) * self.x*self.y
        return

    def energy(self, px, py, pz):  # calculate the field energy at px,py,pz
        lbx = 0.
        lby = 0.
        lbz = 0.
        for ix in (-1, 1):
            xa = int((ix+1)//2)
            for iy in (-1, 1):
                ya = int((iy+1)//2)
                for iz in (-1, 1):
                    za = int((iz+1)//2)
                    rx = px-self.x[za][ya][xa]
                    ry = py-self.y[za][ya][xa]
                    rz = pz-self.z[za][ya][xa]
                    r2 = pow(rx, 2)+pow(ry, 2)+pow(rz, 2)
                    scpr = self.u[za][ya][xa]*rx+self.v[za][ya][xa]*ry+self.w[za][ya][xa]*rz
                    lbx += (3.*scpr*rx - r2*self.u[za][ya][xa])/pow(r2,2.5)
                    lby += (3.*scpr*ry - r2*self.v[za][ya][xa])/pow(r2,2.5)
                    lbz += (3.*scpr*rz - r2*self.w[za][ya][xa])/pow(r2,2.5)
        return np.sqrt(lbx*lbx+lby*lby+lbz*lbz)

    def radial_flux(self, px, py, pz):  # calculate field energy at px,py,pz
        lbx = 0.
        lby = 0.
        lbz = 0.
        for ix in (-1, 1):
            xa = int((ix+1)//2)
            for iy in (-1, 1):
                ya = int((iy+1)//2)
                for iz in (-1, 1):
                    za = int((iz+1)//2)
                    rx = px-self.x[za][ya][xa]
                    ry = py-self.y[za][ya][xa]
                    rz = pz-self.z[za][ya][xa]
                    r2 = pow(rx, 2)+pow(ry, 2)+pow(rz, 2)
                    scpr=self.u[za][ya][xa]*rx+self.v[za][ya][xa]*ry+self.w[za][ya][xa]*rz
                    lbx += (3.*scpr*rx - r2*self.u[za][ya][xa])/pow(r2, 2.5)
                    lby += (3.*scpr*ry - r2*self.v[za][ya][xa])/pow(r2, 2.5)
                    lbz += (3.*scpr*rz - r2*self.w[za][ya][xa])/pow(r2, 2.5)
        return (lbx*px+lby*py+lbz*pz)/np.sqrt(px*px+py*py+pz*pz)

    def potential(self, px, py, pz):  # calculate the potential at px, py, pz
        pot = 0.
        for ix in (-1, 1):
            xa = int((ix+1)//2)
            for iy in (-1, 1):
                ya = int((iy+1)//2)
                for iz in (-1, 1):
                    za = int((iz+1)//2)
                    rx = px-self.x[za][ya][xa]
                    ry = py-self.y[za][ya][xa]
                    rz = pz-self.z[za][ya][xa]
                    r2 = pow(rx, 2)+pow(ry, 2)+pow(rz, 2)
                    scpr = self.u[za][ya][xa]*rx+self.v[za][ya][xa]*ry+self.w[za][ya][xa]*rz
                    pot += scpr/pow(r2, 1.5)
        return pot

    def ind_x(self, px, py, pz, direction):  # calculate the field component
        # along dir at position px,py,pz
        lbx = 0.
        lby = 0.
        lbz = 0.
        for ix in (-1, 1):
            xa = int((ix+1)/2)
            for iy in (-1, 1):
                ya = int((iy+1)/2)
                for iz in (-1, 1):
                    za = int((iz+1)//2)
                    rx = px-self.x[za][ya][xa]
                    ry = py-self.y[za][ya][xa]
                    rz = pz-self.z[za][ya][xa]
                    r2 = pow(rx, 2)+pow(ry, 2)+pow(rz, 2)
                    scpr = self.u[za][ya][xa]*rx+self.v[za][ya][xa]*ry+self.w[za][ya][xa]*rz  # scpr= scalar product
                    lbx += (3.*scpr*rx - r2*self.u[za][ya][xa])/pow(r2, 2.5)
                    lby += (3.*scpr*ry - r2*self.v[za][ya][xa])/pow(r2, 2.5)
                    lbz += (3.*scpr*rz - r2*self.w[za][ya][xa])/pow(r2, 2.5)
        return (lbx*direction[0]+lby*direction[1]+lbz*direction[2])

    def b_feld(self, px, py, pz):  # calculate the field components at px,py,pz
        lbx = 0.
        lby = 0.
        lbz = 0.
        for ix in (-1, 1):
            xa = int((ix+1)/2)
            for iy in (-1, 1):
                ya = int((iy+1)/2)
                for iz in (-1, 1):
                    za = int((iz+1)/2)
                    rx = px-self.x[za][ya][xa]
                    ry = py-self.y[za][ya][xa]
                    rz = pz-self.z[za][ya][xa]
                    r2 = pow(rx, 2)+pow(ry, 2)+pow(rz, 2)
                    scpr=self.u[za][ya][xa]*rx+self.v[za][ya][xa]*ry+self.w[za][ya][xa]*rz
                    lbx += (3.*scpr*rx - r2*self.u[za][ya][xa])/pow(r2, 2.5)
                    lby += (3.*scpr*ry - r2*self.v[za][ya][xa])/pow(r2, 2.5)
                    lbz += (3.*scpr*rz - r2*self.w[za][ya][xa])/pow(r2, 2.5)
        return (lbx, lby, lbz)

    def fix_pot(self, px, py, pz): #potential of a dipole fixed in orientation
        lbx, lby, lbz=self.b_feld(px, py, pz)
        return lbx*1+lby*1+lbz*1

    def analytic(self, px):  # analytic expression for the asymptote
        return 105.*np.sqrt(3./2.)*4.*pow(px*np.sqrt(2.), -7)*np.sin(self.t)